from solver.nonlinear_iterative import NonlinearIterative
import numpy as np
from dolfin import *
import time 
class Newton(NonlinearIterative):

    def __init__(self,pde,step_size_control,max_iter=15,epsi=1e-10,inner_solver="mumps",calc_norm=False,output_file = 'output',ref_solution=None):
        super().__init__(pde,step_size_control,max_iter,epsi,inner_solver,calc_norm,output_file,ref_solution)

    def solve(self):
        # This is the Newton solver
        # Calculates diagnostic variables like norm etc.
        self.calculate_diagnostic_variables()
        for iter in range(self.max_iter):
            print('iter',iter)
            start = time.time()
            # We calculate the solution for Newton's method
            a = self.equation.a_newton()
            L = self.equation.L_newton()
            U_Newton = Function(self.equation.W)
            solve(a == L, U_Newton, self.equation.bc, solver_parameters={"linear_solver": self.inner_solver})

            # We save the old velocity field to check later, if a stopping criteria is fulfilled
            U_old = Function(self.equation.W)
            U_old.assign(self.equation.U)

            # We update the field self.equation.U with the new value
            # We pass information like the equation and the minimization term
            self.step_size_control.calc_step_size(U_Newton,self)
            # We perform diagnostic calculations.
            # Please modify this method in nonlinear_iterative.py corresponding to your needs
            end = time.time()
            # We do not consider the time for calculating diagnostics.
            print('time one iteration',end-start)
            self.diagnostic_stop()

            '''
            u_newton,p_newton = U_Newton.split()
            coordinates = self.equation.geometry.mesh.coordinates()
            P2 = VectorElement("Lagrange", self.equation.geometry.mesh.ufl_cell(), 2)
            P1 = FiniteElement("Lagrange", self.equation.geometry.mesh.ufl_cell(), 1)
            Function_space = FunctionSpace(self.equation.geometry.mesh,P1)
            Du = sym(nabla_grad(u_newton))
            trDu = tr(Du*Du)
            #print('value',trDu(coordinates[0][:]))
            #gradu = project(tr(Du*Du),Function_space)
            #U_help = Function(self.equation.W)
            #u0_help,p0_help = U_help.split()
            u0_help = Function(Function_space)
            #(u_help,p_help) = TrialFunctions(self.equation.W)
            #(v_help,q_help) = TestFunctions(self.equation.W)
            u_help = TrialFunctions(Function_space)
            v_help = TestFunctions(Function_space)

            a_help = u_help*v_help*dx
            L_help = trDu*v_help*dx
            solve(a_help == L_help, u0_help)
            #u0_help,p0_help = U_help.split()
            print('sol',u0_help(coordinates[0][:]))

            gradu = project(tr(Du*Du))
            for j in range(0,np.shape(coordinates)[0]):
                Dv_actual = gradu(coordinates[j][:])
                if(j==0):
                    Dv_min = Dv_actual
                    Dv_max = Dv_actual
            
                if(Dv_actual<Dv_min):
                    Dv_min = Dv_actual

                if(Dv_actual>Dv_max):
                    Dv_max = Dv_actual
            print('min Dv',Dv_min)
            print('max Dv',Dv_max)
            '''

        '''
        # We calculate the minimum and the maximum values of 0.5|Dv|^2
        u_newton,p_newton = U_Newton.split()
        coordinates = self.equation.geometry.mesh.coordinates()
        P2 = VectorElement("Lagrange", self.equation.geometry.mesh.ufl_cell(), 2)
        P1 = FiniteElement("Lagrange", self.equation.geometry.mesh.ufl_cell(), 1)
        Function_space = FunctionSpace(self.equation.geometry.mesh,P1)
        Du = sym(nabla_grad(u_newton))
        gradu = project(tr(Du*Du),Function_space)
        #print('values')
        #print('shape',np.shape(gradu))
        #print(gradu(coordinates[0][:]))
        #print(gradu(coordinates[1][:]))
        
        for j in range(0,np.shape(coordinates)[0]):
            Dv_actual = gradu(coordinates[j][:])
            if(j==0):
                Dv_min = Dv_actual
                Dv_max = Dv_actual
            
            if(Dv_actual<Dv_min):
                Dv_min = Dv_actual

            if(Dv_actual>Dv_max):
                Dv_max = Dv_actual
        print('min Dv',Dv_min)
        print('max Dv',Dv_max)
        '''

        self.equation.save_data(self.output_file,self.norm_list,self.rel_error,self.rel_local_error)
        return self.equation.U
